#include "MainController.h"

MainController::MainController()
    :ch1(p18,0.047,0.081), // yaw
     ch2(p17,0.047,0.081), // pitch
     ch3(p15,0.047,0.081), // amplitude
     ch4(p30,0.047,0.081), // adj
     ch6(p16,0.047,0.081), // frequency
     ch5(p29,0.047,0.081), // manual vs auto control
     syren(p9,p10), //syren is 38400bps serial
     //ap(p25, p26)//,
     leftservo(p21),
     rightservo(p23),
     pcam(p11,p12,p13,10),
     controlThread(&MainController::controlThreadStarter, this, osPriorityRealtime, 1024),
     trackerThread(&MainController::trackerThreadStarter, this, osPriorityHigh, 1024)
{
    wait_ms(50);
    amp = 0.0;
    frq = 1.0;
    frqCmd = frq;
    yaw = 0.5;
    pitch = 0.5;
    adj = 0.5;
    frqMin = 0.8; //hardcoded
    frqMax = 2.9; //hardcoded
    //change = 0;
    //state = 0;
    fullCycle = true;
    volume = 0.0;
    volMax = 0.1;
    volChg = 0.0;
    raiser = 0.0;
    pitAvg = 0.0;
    alPi = 0.2/(0.2+0.001);//tf/(tf+Ts);
    syren.baud(38400);
    //pcam=new pixySPI(p11,p12,p13,10);

    //leftservo.calibrate(0.0008, 45);
    //rightservo.calibrate(-0.0001, 45);

}

MainController::~MainController()
{
    //delete pcam;
}

void MainController::controlThreadStarter(void const *p)
{
    MainController *instance = (MainController*)p;
    instance->control();
}

void MainController::trackerThreadStarter(void const *p)
{
    MainController *instance = (MainController*)p;
    instance->trackTarget();
}

void MainController::control()
{
    controlThread.signal_wait(START_THREAD);
    while (1) {
        trackMutex.lock();
        curTime = timer1.read();
        //Block bestObject=pcam.getBlocks()[0]; //our best bet is the first block because it has the biggest area

        // check every half cycle
        if(curTime > 1/(2*frqCmd) ) {
            // read new yaw value every half cycle
            if (ch5.dutycyclescaledup()<=0.5)
                yaw = this->calculateYaw(); // a value from -1 to 1
            else {
                //do proportional control on fish yaw
                float gain=1/float(CENTER_X);
                float error=pcam.getBestX()-CENTER_X; //calculate yaw difference
                yaw=error*gain;
                //printf("xlocation: %d -> %f\n", pcam.getBestX(), yaw);
            }

            if(yaw < 0.1 && yaw > -0.1) {
                yaw =0.0;
            }
            // calculate liquid volume change in the chambers
            volChg = volMax * yaw;
            volChg = 0.0;

            timeAdd = 0.0;

            // Read volume and frequency only every full cycle
            if( fullCycle ) {
                //read other new inputs
                amp = this->calculateAmplitude(); // a value from 0 to 1
                frq = this->calculateFrequency(); // a value from frqmin to frqmax

                if(volChg > 0.0) {
                    // adjust frequency to add additional volume
                    if( amp < 0.0001 ) {
                        amp = 0.0001;
                    }
                    timeAdd = volChg/amp;

                    if( timeAdd > 0.5/frq ) {
                        timeAdd = 0.5/frq;
                        volChg = timeAdd * amp;
                    }
                }
                ampNew = amp;

                if(yaw < 0.0) {
                    ampNew = (1.0+0.7*yaw)*amp;
                }

                fullCycle = false;

            } else {
                // reverse for the downward slope
                amp = -amp;

                if(volChg < 0.0) {
                    // adjust frequency to add additional volume
                    if( amp > -0.0001 ) {
                        amp = -0.0001;
                    }
                    timeAdd = volChg/amp;

                    if( timeAdd > 0.5/frq ) {
                        timeAdd = 0.5/frq;
                        volChg = timeAdd * amp;
                    }
                }

                ampNew = amp;

                if(yaw > 0.0) {
                    ampNew = (1.0-0.7*yaw)*amp;
                }

                // use amp and frq from last cycle in order to make sure it is equalized
                fullCycle = true;
            }
            // update the frequency that actually needs to be commanded
            frqCmd = 1.0/( 2.0*( timeAdd + 1/( 2* frq) ) );

            // read new yaw value every half cycle
            adj = this->calculateAdj(); // a value from 0 to 1

            // for keeping track, calculate current volume storage, positive means on side is fuller, negative means other side is fuller
            //volume = volChg + volume;
            // rudder value used to define the trapezoid shape
            raiser = ( 5 * adj + 1.0); // varied from 1 to 5

            //reset timer
            timer1.reset();
            curTime = 0.0;
        }


        //Set Servo Values
        if (ch5.dutycyclescaledup()<=0.5)
            pitch = this->calculatePitch();
        else {
            //do a proportional control on the error since servos already give you position control
            float gain=1/float(CENTER_Y); //try this out for now
            float error=pcam.getBestY()-CENTER_Y;
            pitch=error*gain;
            //printf("ylocation: %d -> %f\n", pcam.getBestY(), pitch);
        }
        leftservo = pitch+0.03;
        rightservo = 1.0 - pitch;
        if (rightservo<0.03) {
            rightservo = 0.03;
        }

        dutyCycle = ampNew * saturate(raiser * sin( 2.0 * MATH_PI * frqCmd * curTime )); // add factor 4.0 to get a cut off sinus

        //now scale duty cycle from -1 to 1 -> 0 to 255
        dutyCycle=(255/2)*dutyCycle+127;
        if (dutyCycle<0)
            dutyCycle=0.0;
        else if (dutyCycle>255)
            dutyCycle=255.0;

        //mcon.setpolarspeed(dutyCycle);
        syren.putc(int(dutyCycle));
        //syren.putc(int(0));
        trackMutex.unlock();
        Thread::wait(1);
    }
}

void MainController::trackTarget()
{
    trackerThread.signal_wait(START_THREAD);
    while (1) {
        //trackMutex.lock();
        //printf("tracking\n");
        pcam.capture();
        //trackMutex.unlock();
    }
}

float MainController::calculateFrequency()
{
    return ((frqMax - frqMin) * ch6.dutycyclescaledup() + frqMin);
}

float MainController::calculateAmplitude()
{
    return (ch3.dutycyclescaledup());
}

float MainController::calculateYaw()
{
    return (2.0*ch1.dutycyclescaledup() - 1.0);
}

float MainController::calculatePitch()
{
    pitAvg = alPi * pitAvg+ (1.0 - alPi)*(ch2.dutycyclescaledup());
    return pitAvg;
}

float MainController::calculateAdj()
{
    return (ch4.dutycyclescaledup());
}

void MainController::start()
{
    timer1.start();

    printf("start thread\n");
    controlThread.signal_set(START_THREAD);
    trackerThread.signal_set(START_THREAD);
    //ticker1.attach(this, &MainController::control,0.001);
    //control=Thread(&MainController::control);
    //tracker.attach(this, &MainController::trackTarget, 2.00); //update target position every second
    //Autopilot guardian
    //ap.calibrate();
    //ap.set2D();
    //ap.setoff();

}

void MainController::stop()
{
    timer1.stop();
    //ticker1.detach();
    //tracker.detach();
    //mcon.setpolarspeed(0.0);
    syren.putc(int(127));
}

float MainController::getDutyCycle()
{
    return dutyCycle;
}

float MainController::getAmplitude()
{
    return amp;
}


float MainController::getFrequency()
{
    return frq;
}

float MainController::getVolume()
{
    return volume;
}

float MainController::getYaw()
{
    return yaw;
}

float MainController::getPitch()
{
    return pitch;
}

float MainController::getAdj()
{
    return adj;
}

float MainController::getTimeAdd()
{
    return timeAdd;
}

bool MainController::getOpMode()
{
    return ch5.dutycyclescaledup()>0.5;
}

//signum function
float MainController::signum(float input)
{
    if (input>0.0)
        return 1.0;
    else if (input<0.0)
        return (-1.0);
    else
        return 0.0;
}

//saturate function
float MainController::saturate(float input)
{
    if (input > 1.0)
        return (1.0);
    else if (input < -1.0)
        return (-1.0);
    else
        return input;
}
